<?php

namespace Drupal\tmgmt_ec_etranslation\Plugin\tmgmt\Translator;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\tmgmt\ContinuousTranslatorInterface;
use Drupal\tmgmt\JobInterface;
use Drupal\tmgmt\JobItemInterface;
use Drupal\tmgmt\TranslatorInterface;
use Drupal\tmgmt\TranslatorPluginBase;
use GuzzleHttp\Exception\GuzzleException;
use GuzzleHttp\RequestOptions;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * TMGMT Translator for eTranslation.
 *
 * @TranslatorPlugin(
 *   id = \Drupal\tmgmt_ec_etranslation\Plugin\tmgmt\Translator\ETranslationTranslator::ID,
 *   label = @Translation("European Commission eTranslation"),
 *   description = @Translation("Allows the European Commission's eTranslation service to process translation jobs."),
 *   ui = "\Drupal\tmgmt_ec_etranslation\ETranslationTranslatorUi",
 * )
 */
class ETranslationTranslator extends TranslatorPluginBase implements ContainerFactoryPluginInterface, ContinuousTranslatorInterface {

  public const ID = 'ec_etranslation';

  /**
   * A cache bin for domain info.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected $cache;

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;

  /**
   * Data related functions for TMGMT.
   *
   * @var \Drupal\tmgmt\Data
   */
  protected $tmgmtData;

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id, $plugin_definition
  ) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->cache = $container->get('cache.tmgmt_ec_etranslation');
    $instance->logger = $container->get('logger.channel.tmgmt_ec_etranslation');
    $instance->client = $container->get('tmgmt_ec_etranslation.client');
    $instance->tmgmtData = $container->get('tmgmt.data');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultSettings() {
    return parent::defaultSettings() + [
      'username' => NULL,
      'password' => NULL,
      'domain' => 'SPD',
      'email' => NULL,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function requestTranslation(JobInterface $job) {
    $this->requestJobItemsTranslation($job->getItems());
    if (!$job->isRejected()) {
      $job->submitted();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function requestJobItemsTranslation(array $job_items) {
    /** @var \Drupal\tmgmt\JobItemInterface[] $job_items */
    foreach ($job_items as $item) {
      $job = $item->getJob();
      if ($job->isContinuous()) {
        $item->active();
      }

      $callerInformation = [
        'application' => 'SOAP_UI',
        'username' => 'eyi',
      ];

      $translatableFields = $this->tmgmtData->filterTranslatable($item->getData());

      foreach ($translatableFields as $key => $field) {
        $translator = $job->getTranslator();

        $data = [
          'sourceLanguage' => strtoupper($job->getRemoteSourceLanguage()),
          'externalReference' => $key,
          'domain' => $translator->getSetting('domain'),
          'callerInformation' => $callerInformation,
          'targetLanguages' => [strtoupper($job->getRemoteTargetLanguage())],
          'requesterCallback' => Url::fromRoute('tmgmt_ec_etranslation.success', ['jobItem' => $item->id()])->setAbsolute()->toString(),
          'errorCallback' => Url::fromRoute('tmgmt_ec_etranslation.error', ['jobItem' => $item->id()])->setAbsolute()->toString(),
          'destinations' => [
            'httpDestinations' => [
              Url::fromRoute('tmgmt_ec_etranslation.callback', ['jobItem' => $item->id()])->setAbsolute()->toString(),
            ],
          ],
        ];

        if ($email = $translator->getSetting('email')) {
          $data['destinations']['emailDestinations'] = [$email];
        }

        if ($this->isHtml($field['#text'])) {
          $data['documentToTranslateBase64'] = [
            'content' => base64_encode($field['#text']),
            'format' => 'html',
          ];
        }
        else {
          $data['textToTranslate'] = $field['#text'];
        }

        try {
          $this->client->post('etranslation/si/translate', [
            RequestOptions::AUTH => [
              $translator->getSetting('username'),
              $translator->getSetting('password'),
              'digest',
            ],
            RequestOptions::JSON => $data,
          ]);
        }
        catch (GuzzleException $e) {
          $message = $this->t('Job item has been rejected with following error: @error.', ['@error' => $e->getMessage()]);
          $item->setState(JobItemInterface::STATE_INACTIVE, $message);

          return;
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedLanguagePairs(TranslatorInterface $translator): array {
    $domain = $translator->getSetting('domain');
    $cacheKey = implode(':', [$translator->id(), 'language_pairs', $domain]);

    if ($cache = $this->cache->get($cacheKey)) {
      return $cache->data;
    }

    $data = $this->getDomainInfo($translator);
    if ($data === []) {
      // Try again later.
      return [];
    }

    if (!isset($data[$domain])) {
      // Don't try again later.
      $this->cache->set($cacheKey, []);
      return [];
    }

    $pairs = [];
    foreach ($data[$domain]['languagePairs'] as $languagePair) {
      [$sourceLangcode, $targetLangcode] = explode('-', $languagePair);

      $pairs[] = [
        'source_language' => strtolower($sourceLangcode),
        'target_language' => strtolower($targetLangcode),
      ];
    }

    $this->cache->set($cacheKey, $pairs);
    return $pairs;
  }

  /**
   * {@inheritdoc}
   */
  public function getSupportedTargetLanguages(TranslatorInterface $translator, $source_language) {
    $languages = [];

    foreach ($this->getSupportedLanguagePairs($translator) as $pair) {
      if ($pair['source_language'] === $source_language) {
        $languages[$pair['target_language']] = $pair['target_language'];
      }
    }

    return $languages;
  }

  /**
   * Fetch domain info from the API.
   */
  protected function getDomainInfo(TranslatorInterface $translator): array {
    $cacheKey = implode(':', [$translator->id(), 'domain_info']);

    if ($cache = $this->cache->get($cacheKey)) {
      return $cache->data;
    }

    try {
      $response = $this->client->get('etranslation/si/get-domains', [
        RequestOptions::AUTH => [
          $translator->getSetting('username'),
          $translator->getSetting('password'),
          'digest',
        ],
      ]);
    }
    catch (GuzzleException $e) {
      $this->logger->error($this->t('Unable to get domain info: @error', ['@error' => $e->getMessage()]));
      return [];
    }

    $contentType = $response->getHeader('Content-Type')[0];
    if (strpos($contentType, 'application/json') !== 0) {
      $this->logger->error($this->t('Unable to get domain info: invalid response format'));
      return [];
    }

    $data = json_decode($response->getBody()->getContents(), TRUE);
    $this->cache->set($cacheKey, $data);

    return $data;
  }

  /**
   * Checks if the given text is HTML.
   */
  protected function isHtml(string $text): bool {
    return \strip_tags($text) !== $text;
  }

}
